import { GridEventDispatcher } from './utils';
import { createDeepOnChangeProxy } from './utils/createDeepOnChangeProxy';
/**
 * A Cell is part of a grid. It contains meta information like its coordinates inside the grid and the corresponding value.
 */
export class Cell {
    id;
    row;
    col;
    valueProxy;
    eventDispatcher;
    /**
     * @param coordinate The coordinate in the grid
     * @param value The value of the cell
     */
    constructor(coordinate, value) {
        this.id = `cell|${coordinate.row}-${coordinate.col}`;
        this.row = coordinate.row;
        this.col = coordinate.col;
        this.valueProxy =
            value && typeof value === 'object'
                ? createDeepOnChangeProxy(value, () => this.eventDispatcher.dispatchCellValueChangedEvent({ cell: this }))
                : value;
        this.eventDispatcher = new GridEventDispatcher();
    }
    /**
     * @returns the value of the cell
     */
    get value() {
        return this.valueProxy;
    }
    /**
     * Changes the value of the cell
     */
    set value(value) {
        this.valueProxy = value;
        this.eventDispatcher.dispatchCellValueChangedEvent({ cell: this });
    }
    /**
     * @param callback A function that should be called, when the cell value changes
     * @returns a function to unregister the callback
     */
    onValueChanged(callback) {
        return this.eventDispatcher.onCellValueChanged(callback);
    }
}
